@@ -22,13 +22,13 @@
 #include <language/codecompletion/codecompletionmodel.h>
 #include <language/duchain/types/functiontype.h>
 #include <language/duchain/aliasdeclaration.h>
+#include <language/duchain/types/containertypes.h>
 
 #include <KTextEditor/View>
 #include <KTextEditor/Document>
 #include <KLocalizedString>
 
 #include "duchain/navigation/navigationwidget.h"
-#include "types/variablelengthcontainer.h"
 #include "codecompletion/helpers.h"
 #include "declaration.h"
 #include "declarations/functiondeclaration.h"
@@ -71,8 +71,8 @@ int FunctionDeclarationCompletionItem::argumentHintDepth() const
 
 QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const
 {
-    FunctionDeclaration* dec = dynamic_cast<FunctionDeclaration*>(m_declaration.data());
     DUChainReadLocker lock;
+    FunctionDeclaration* dec = dynamic_cast<FunctionDeclaration*>(m_declaration.data());
     switch ( role ) {
         case Qt::DisplayRole: {
             if ( ! dec ) {
@@ -86,7 +86,8 @@ QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int r
                 }
             }
             if ( index.column() == KDevelop::CodeCompletionModel::Prefix ) {
-                if ( FunctionType::Ptr type = dec->type<FunctionType>() ) {
+                FunctionType::Ptr type = dec->type<FunctionType>();
+                if ( type && type->returnType() ) {
                     return i18n("function") + " -> " + type->returnType()->toString();
                 }
             }
@@ -114,7 +115,7 @@ QVariant FunctionDeclarationCompletionItem::data(const QModelIndex& index, int r
         case KDevelop::CodeCompletionModel::MatchQuality: {
             if (    m_typeHint == PythonCodeCompletionContext::IterableRequested
                  && dec && dec->type<FunctionType>()
-                 && dynamic_cast<VariableLengthContainer*>(dec->type<FunctionType>()->returnType().unsafeData()) )
+                 && dynamic_cast<ListType*>(dec->type<FunctionType>()->returnType().unsafeData()) )
             {
                 return 2 + PythonDeclarationCompletionItem::data(index, role, model).toInt();
             }
@@ -154,7 +155,7 @@ void FunctionDeclarationCompletionItem::executed(KTextEditor::Document* document
     }
     // place cursor behind bracktes by default
     int skip = 2;
-    if ( fdecl.first ) {
+    if ( fdecl.first && fdecl.first->type<FunctionType>() ) {
         bool needsArguments = false;
         int argumentCount = fdecl.first->type<FunctionType>()->arguments().length();
         if ( fdecl.first->context()->type() == KDevelop::DUContext::Class ) {
