@@ -64,14 +64,14 @@ ReferencedTopDUContext ContextBuilder::build(const IndexedString& url, Ast* node
         }
     }
     if (updateContext) {
-        qDebug() << " ====> DUCHAIN ====>     rebuilding duchain for" << url.str() << "(was built before)";
+        kDebug() << " ====> DUCHAIN ====>     rebuilding duchain for" << url.str() << "(was built before)";
         DUChainWriteLocker lock(DUChain::lock());
         Q_ASSERT(updateContext->type() == DUContext::Global);
         updateContext->clearImportedParentContexts();
         updateContext->parsingEnvironmentFile()->clearModificationRevisions();
         updateContext->clearProblems();
     } else {
-        qDebug() << " ====> DUCHAIN ====>     building duchain for" << url.str();
+        kDebug() << " ====> DUCHAIN ====>     building duchain for" << url.str();
     }
     return ContextBuilderBase::build(url, node, updateContext);
 }
@@ -244,8 +244,6 @@ void ContextBuilder::visitGeneratorExpression(GeneratorExpressionAst* node)
 RangeInRevision ContextBuilder::comprehensionRange(Ast* node)
 {
     RangeInRevision range = editorFindRange(node, node);
-    range.start.column -= 1;
-    range.end.column += 1;
     return range;
 }
 
