@@ -22,7 +22,7 @@
 #include "duchain/declarations/decorator.h"
 #include "duchain/declarations/functiondeclaration.h"
 #include "duchain/declarations/classdeclaration.h"
-#include "types/variablelengthcontainer.h"
+
 #include "types/hintedtype.h"
 #include "types/unsuretype.h"
 #include "types/indexedcontainer.h"
@@ -31,6 +31,7 @@
 #include "pythoneditorintegrator.h"
 #include "helpers.h"
 #include "assistants/missingincludeassistant.h"
+#include "correctionhelper.h"
 
 #include <language/duchain/functiondeclaration.h>
 #include <language/duchain/declaration.h>
@@ -57,9 +58,9 @@ using namespace KDevelop;
 namespace Python
 {
 
-DeclarationBuilder::DeclarationBuilder(PythonEditorIntegrator* editor)
-        : DeclarationBuilderBase()
-        , m_ownPriority(0)
+DeclarationBuilder::DeclarationBuilder(Python::PythonEditorIntegrator* editor, int ownPriority)
+    : DeclarationBuilderBase()
+    , m_ownPriority(ownPriority)
 {
     setEditor(editor);
     kDebug() << "Building Declarations";
@@ -104,6 +105,11 @@ ReferencedTopDUContext DeclarationBuilder::build(const IndexedString& url, Ast*
     return DeclarationBuilderBase::build(url, node, updateContext);
 }
 
+int DeclarationBuilder::jobPriority() const
+{
+    return m_ownPriority;
+}
+
 void DeclarationBuilder::closeDeclaration()
 {
     if ( lastContext() ) {
@@ -315,12 +321,12 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Identifier*
             // check for argument type hints (those are created when calling functions)
             AbstractType::Ptr hints = Helper::extractTypeHints(dec->abstractType(), topContext());
             kDebug() << hints->toString();
-            if ( hints.cast<IndexedContainer>() || hints.cast<VariableLengthContainer>() ) {
+            if ( hints.cast<IndexedContainer>() || hints.cast<ListType>() ) {
                 // This only happens when the type hint is a tuple, which means the vararg/kwarg of a function is being processed.
                 newType = hints;
             }
             else {
-                newType = Helper::mergeTypes(hints, type, topContext());
+                newType = Helper::mergeTypes(hints, type);
             }
         }
         else {
@@ -338,7 +344,7 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Identifier*
         if ( newType ) {
             if ( currentType && currentType->indexed() != newType->indexed() ) {
                 // If the previous and new type are different, use an unsure type
-                dec->setType(Helper::mergeTypes(currentType, newType, topContext()));
+                dec->setType(Helper::mergeTypes(currentType, newType));
             }
             else {
                 // If no type was set previously, use only the new one.
@@ -374,7 +380,7 @@ void DeclarationBuilder::visitWithItem(WithItemAst* node)
 {
     if ( node->optionalVars ) {
         // For statements like "with open(f) as x", a new variable must be created; do this here.
-        ExpressionVisitor v(currentContext(), editor());
+        ExpressionVisitor v(currentContext());
         v.visitNode(node->contextExpression);
         visitVariableDeclaration<Declaration>(node->optionalVars, 0, v.lastType());
     }
@@ -383,19 +389,19 @@ void DeclarationBuilder::visitWithItem(WithItemAst* node)
 
 void DeclarationBuilder::visitFor(ForAst* node)
 {
-    ExpressionVisitor v(currentContext(), editor());
+    ExpressionVisitor v(currentContext());
     v.visitNode(node->iterator);
-    QList<VariableLengthContainer::Ptr> possibleIterators = Helper::filterType<VariableLengthContainer>(v.lastType(),
+    auto possibleIterators = Helper::filterType<ListType>(v.lastType(),
         [](AbstractType::Ptr type) {
-            auto container = type.cast<VariableLengthContainer>();
+            auto container = type.cast<ListType>();
             return container && container->contentType();
         }
     );
     if ( node->target->astType == Ast::NameAstType ) {
         // In case the iterator variable is a Name ("for x in range(3)"), just create a declaration for it.
         // The following code tries to figure out the type of "x" from the object that is being iterated over.
-        auto iteratorType = Helper::foldTypes<VariableLengthContainer::Ptr>(possibleIterators,
-            [](const VariableLengthContainer::Ptr& p) {
+        auto iteratorType = Helper::foldTypes<ListType::Ptr>(possibleIterators,
+            [](const ListType::Ptr& p) {
                 return p->contentType().abstractType();
             }
         );
@@ -407,10 +413,10 @@ void DeclarationBuilder::visitFor(ForAst* node)
     else if ( node->target->astType == Ast::TupleAstType ) {
         // If the target is a tuple ("for x, y, z in ..."), multiple variables must be declared.
         // For now, types of those variables will only be determined if the iterator is a list of tuples.
-        QList<ExpressionAst*> targetElements = targetsOfAssignment(QList<ExpressionAst*>{node->target});
+        QList<ExpressionAst*> targetElements = targetsOfAssignment(QList<ExpressionAst*>() << node->target);
         int targetElementsCount = targetElements.count();
         QList<IndexedContainer::Ptr> gatherFromTuples;
-        foreach ( VariableLengthContainer::Ptr container, possibleIterators ) {
+        for ( auto container : possibleIterators ) {
             AbstractType::Ptr contentType = container->contentType().abstractType();
             gatherFromTuples = Helper::filterType<IndexedContainer>(contentType,
                 // find all IndexedContainer entries which have the right number of entries
@@ -483,7 +489,8 @@ Declaration* DeclarationBuilder::findDeclarationInContext(QStringList dottedName
     return lastAccessedDeclaration;
 }
 
-QString DeclarationBuilder::buildModuleNameFromNode(ImportFromAst* node, AliasAst* alias, const QString& intermediate) {
+QString DeclarationBuilder::buildModuleNameFromNode(ImportFromAst* node, AliasAst* alias, const QString& intermediate) const
+{
     QString moduleName = alias->name->value;
     if ( ! intermediate.isEmpty() ) {
         moduleName.prepend('.').prepend(intermediate);
@@ -491,7 +498,7 @@ QString DeclarationBuilder::buildModuleNameFromNode(ImportFromAst* node, AliasAs
     if ( node->module ) {
         moduleName.prepend('.').prepend(node->module->value);
     }
-    // To handle relative import correct, add node level in the beginning of the path
+    // To handle relative imports correctly, add node level in the beginning of the path
     // This will allow findModulePath to deduce module search direcotry properly
     moduleName.prepend(QString(node->level, '.'));
     return moduleName;
@@ -545,6 +552,7 @@ void DeclarationBuilder::visitComprehension(ComprehensionAst* node)
     // TODO add a special case to the usebuilder to display the second occurence as a declaration
     RangeInRevision declarationRange(currentContext()->range().start, currentContext()->range().start);
     declarationRange.end.column -= 1;
+    declarationRange.start.column -= 1;
     
     AbstractType::Ptr targetType(new IntegralType(IntegralType::TypeMixed));
     if ( node->iterator ) {
@@ -552,7 +560,7 @@ void DeclarationBuilder::visitComprehension(ComprehensionAst* node)
         // type of the iterator variable
         ExpressionVisitor v(currentContext());
         v.visitNode(node->iterator);
-        if ( VariableLengthContainer* container = dynamic_cast<VariableLengthContainer*>(v.lastType().unsafeData()) ) {
+        if ( auto container = ListType::Ptr::dynamicCast(v.lastType()) ) {
             targetType = container->contentType().abstractType();
         }
     }
@@ -566,15 +574,6 @@ void DeclarationBuilder::visitComprehension(ComprehensionAst* node)
             if ( tupleElt->astType == Ast::NameAstType ) {
                 NameAst* n = static_cast<NameAst*>(tupleElt);
                 visitVariableDeclaration<Declaration>(n->identifier, declarationRange);
-                // TODO: Fix this as soon as tuple type support is implemented.
-//                 DUChainWriteLocker lock(DUChain::lock());
-//                 d->setAbstractType(AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed)));
-//                 if ( d and targetType ) {
-//                     d->setAbstractType(targetType);
-//                 }
-//                 else {
-//                     d->setAbstractType(AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed)));
-//                 }
             }
         }
     }
@@ -583,7 +582,7 @@ void DeclarationBuilder::visitComprehension(ComprehensionAst* node)
 void DeclarationBuilder::visitImport(ImportAst* node)
 {
     Python::ContextBuilder::visitImport(node);
-    DUChainWriteLocker lock(DUChain::lock());
+    DUChainWriteLocker lock;
     foreach ( AliasAst* name, node->names ) {
         QString moduleName = name->name->value;
         // use alias if available, name otherwise
@@ -665,7 +664,7 @@ Declaration* DeclarationBuilder::createDeclarationTree(const QStringList& nameCo
     
     RangeInRevision displayRange = RangeInRevision::invalid();
     
-    DUChainWriteLocker lock(DUChain::lock());
+    DUChainWriteLocker lock;
     for ( int i = 0; i < remainingNameComponents.length(); i++ ) {
         // Iterate over all the names, and create a declaration + sub-context for each of them
         const QString& component = remainingNameComponents.at(i);
@@ -778,7 +777,6 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
 {
     // Search the disk for a python file which contains the requested declaration
     QPair<KUrl, QStringList> moduleInfo = findModulePath(moduleName, currentlyParsedDocument().toUrl());
-    kDebug() << moduleName;
     RangeInRevision range(RangeInRevision::invalid());
     if ( rangeNode ) {
         range = rangeForNode(rangeNode, false);
@@ -800,12 +798,11 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
         // a missing module, or a C module (.so) which is unreadable for kdevelop
         // TODO imrpove error handling in case the module exists as a shared object or .pyc file only
         kDebug() << "invalid or non-existent URL:" << moduleInfo;
-        KDevelop::Problem *p = new KDevelop::Problem();
-        p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange())); // TODO ok?
+        KDevelop::Problem *p = new Python::MissingIncludeProblem(moduleName, currentlyParsedDocument());
+        p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange()));
         p->setSource(KDevelop::ProblemData::SemanticAnalysis);
         p->setSeverity(KDevelop::ProblemData::Warning);
         p->setDescription(i18n("Module \"%1\" not found", moduleName));
-        p->setSolutionAssistant(KSharedPtr<IAssistant>(new MissingIncludeAssistant(moduleName, currentlyParsedDocument())));
         problemEncountered.attach(p);
         return 0;
     }
@@ -840,12 +837,11 @@ Declaration* DeclarationBuilder::createModuleImportDeclaration(QString moduleNam
                                                              editorFindRange(declarationIdentifier, declarationIdentifier));
             }
             else {
-                KDevelop::Problem *p = new KDevelop::Problem();
+                KDevelop::Problem *p = new Python::MissingIncludeProblem(moduleName, currentlyParsedDocument());
                 p->setFinalLocation(DocumentRange(currentlyParsedDocument(), range.castToSimpleRange())); // TODO ok?
                 p->setSource(KDevelop::ProblemData::SemanticAnalysis);
                 p->setSeverity(KDevelop::ProblemData::Warning);
                 p->setDescription(i18n("Declaration for \"%1\" not found in specified module", moduleInfo.second.join(".")));
-                p->setSolutionAssistant(KSharedPtr<IAssistant>(new MissingIncludeAssistant(moduleName, currentlyParsedDocument())));
                 problemEncountered.attach(p);
             }
         }
@@ -860,7 +856,7 @@ void DeclarationBuilder::visitYield(YieldAst* node)
     AstDefaultVisitor::visitYield(node);
     
     // Determine the type of the argument to "yield", like "int" in "yield 3"
-    ExpressionVisitor v(currentContext(), editor());
+    ExpressionVisitor v(currentContext());
     v.visitNode(node->value);
     AbstractType::Ptr encountered = v.lastType();
     
@@ -874,19 +870,19 @@ void DeclarationBuilder::visitYield(YieldAst* node)
     if ( ! t ) {
         return;
     }
-    if ( VariableLengthContainer::Ptr previous = t->returnType().cast<VariableLengthContainer>() ) {
+    if ( auto previous = t->returnType().cast<ListType>() ) {
         // If the return type of the function already is set to a list, *add* the encountered type
         // to its possible content types.
-        previous->addContentType(encountered);
+        previous->addContentType<Python::UnsureType>(encountered);
         t->setReturnType(previous.cast<AbstractType>());
     }
     else {
         // Otherwise, create a new container type, and set it as the function's return type.
         DUChainWriteLocker lock;
-        VariableLengthContainer::Ptr container = ExpressionVisitor::typeObjectForIntegralType<VariableLengthContainer>("list", currentContext());
+        auto container = ExpressionVisitor::typeObjectForIntegralType<ListType>("list", currentContext());
         if ( container ) {
-            openType<VariableLengthContainer>(container);
-            container->addContentType(encountered);
+            openType<ListType>(container);
+            container->addContentType<Python::UnsureType>(encountered);
             t->setReturnType(Helper::mergeTypes(t->returnType(), container.cast<AbstractType>()));
             closeType();
         }
@@ -896,7 +892,7 @@ void DeclarationBuilder::visitYield(YieldAst* node)
 void DeclarationBuilder::visitLambda(LambdaAst* node)
 {
     Python::AstDefaultVisitor::visitLambda(node);
-    DUChainWriteLocker lock(DUChain::lock());
+    DUChainWriteLocker lock;
     // A context must be opened, because the lamdba's arguments are local to the lambda:
     // d = lambda x: x*2; print x # <- gives an error
     openContext(node, editorFindRange(node, node->body), DUContext::Other);
@@ -912,7 +908,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
     v.visitNode(static_cast<AttributeAst*>(node->function)->value);
 
     // Don't do anything if the object the function is being called on is not a container.
-    VariableLengthContainer::Ptr container = v.lastType().cast<VariableLengthContainer>();
+    auto container = v.lastType().cast<ListType>();
     if ( ! container || ! function ) {
         return;
     }
@@ -921,8 +917,8 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
         return;
     }
     // Check for the different types of modifiers such a function can have
-    QHash< QString, std::function<void()> > items;
     QStringList args;
+    QHash< QString, std::function<void()> > items;
     items["addsTypeOfArg"] = [&]() {
         const int offset = ! args.isEmpty() ? args.at(0).toInt() : 0;
         if ( node->arguments.length() <= offset ) {
@@ -937,7 +933,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
         }
         DUChainWriteLocker wlock;
         kDebug() << "Adding content type: " << argVisitor.lastType()->toString();
-        container->addContentType(argVisitor.lastType());
+        container->addContentType<Python::UnsureType>(argVisitor.lastType());
         v.lastDeclaration()->setType(container);
     };
     items["addsTypeOfArgContent"] = [&]() {
@@ -951,16 +947,16 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
         if ( ! argVisitor.lastType() ) {
             return;
         }
-        QList<VariableLengthContainer::Ptr> sources = Helper::filterType<VariableLengthContainer>(
+        auto sources = Helper::filterType<ListType>(
             argVisitor.lastType(), [](AbstractType::Ptr type) {
-                return type.cast<VariableLengthContainer>();
+                return type.cast<ListType>();
             }
         );
-        foreach ( VariableLengthContainer::Ptr sourceContainer, sources ) {
+        for ( auto sourceContainer : sources ) {
             if ( ! sourceContainer->contentType() ) {
                 continue;
             }
-            container->addContentType(sourceContainer->contentType().abstractType());
+            container->addContentType<Python::UnsureType>(sourceContainer->contentType().abstractType());
             v.lastDeclaration()->setType(container);
         }
     };
@@ -1065,7 +1061,7 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
             kDebug() << "adding" << argumentType << "at position" << currentArgumentIndex << "/" << currentParamIndex;
             if ( ! argumentType ) continue;
             AbstractType::Ptr newType = Helper::mergeTypes(parameters.at(currentParamIndex)->abstractType(),
-                                                            addType.cast<AbstractType>(), topContext());
+                                                            addType.cast<AbstractType>());
             // TODO this does not correctly update the types in quickopen! Investigate why.
             functiontype->removeArgument(currentArgumentIndex + hasSelfArgument);
             functiontype->addArgument(newType, currentArgumentIndex + hasSelfArgument);
@@ -1082,12 +1078,12 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
     }
     foreach ( KeywordAst* keyword, node->keywords ) {
         AbstractType::Ptr param = parameters.last()->abstractType();
-        VariableLengthContainer::Ptr variable = param.cast<VariableLengthContainer>();
-        if ( ! variable ) {
+        auto list = param.cast<ListType>();
+        if ( ! list ) {
             continue;
         }
         wlock.unlock();
-        ExpressionVisitor argumentVisitor(currentContext(), editor());
+        ExpressionVisitor argumentVisitor(currentContext());
         argumentVisitor.visitNode(keyword->value);
         if ( ! argumentVisitor.lastType() ) {
             continue;
@@ -1098,8 +1094,8 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
         addType->setType(argumentVisitor.lastType());
         addType->setCreatedBy(topContext(), m_futureModificationRevision);
         closeType();
-        variable->addContentType(addType.cast<AbstractType>());
-        parameters.last()->setAbstractType(variable.cast<AbstractType>());
+        list->addContentType<Python::UnsureType>(addType.cast<AbstractType>());
+        parameters.last()->setAbstractType(list.cast<AbstractType>());
     }
 }
 
@@ -1135,7 +1131,7 @@ void DeclarationBuilder::visitCall(CallAst* node)
     addArgumentTypeHints(node, functionVisitor.lastDeclaration());
 }
 
-QList< ExpressionAst* > DeclarationBuilder::targetsOfAssignment(QList< ExpressionAst* > targets)
+QList<ExpressionAst*> DeclarationBuilder::targetsOfAssignment(QList<ExpressionAst*> targets) const
 {
     QList<ExpressionAst*> lhsExpressions;
     foreach ( ExpressionAst* target, targets ) {
@@ -1159,7 +1155,8 @@ QList< ExpressionAst* > DeclarationBuilder::targetsOfAssignment(QList< Expressio
     return lhsExpressions;
 }
 
-QList< DeclarationBuilder::SourceType > DeclarationBuilder::sourcesOfAssignment(ExpressionAst* items, int fillWhenLengthMissing)
+QList<DeclarationBuilder::SourceType> DeclarationBuilder::sourcesOfAssignment(ExpressionAst* items,
+                                                                              int fillWhenLengthMissing) const
 {
     QList<SourceType> sources;
     QList<ExpressionAst*> values;
@@ -1174,7 +1171,7 @@ QList< DeclarationBuilder::SourceType > DeclarationBuilder::sourcesOfAssignment(
         if ( fillWhenLengthMissing > 0 ) {
             ExpressionVisitor v(currentContext());
             v.visitNode(items);
-            auto container = VariableLengthContainer::Ptr::dynamicCast(v.lastType());
+            auto container = ListType::Ptr::dynamicCast(v.lastType());
             if ( container ) {
                 AbstractType::Ptr content = container->contentType().abstractType();
                 for ( ; fillWhenLengthMissing != 0; fillWhenLengthMissing-- ) {
@@ -1195,15 +1192,15 @@ QList< DeclarationBuilder::SourceType > DeclarationBuilder::sourcesOfAssignment(
         sources << SourceType{
             v.lastType(),
             DeclarationPointer(Helper::resolveAliasDeclaration(v.lastDeclaration().data())),
-            v.m_isAlias
+            v.isAlias()
         };
     }
     return sources;
 }
 
-DeclarationBuilder::SourceType DeclarationBuilder::selectSource(const QList< ExpressionAst* >& targets,
-                                                          const QList< DeclarationBuilder::SourceType >& sources,
-                                                          int index, ExpressionAst* value)
+DeclarationBuilder::SourceType DeclarationBuilder::selectSource(const QList<ExpressionAst*>& targets,
+                                                                const QList<DeclarationBuilder::SourceType>& sources,
+                                                                int index, ExpressionAst* rhs) const
 {
     bool canUnpack = targets.length() == sources.length();
     SourceType element;
@@ -1217,11 +1214,11 @@ DeclarationBuilder::SourceType DeclarationBuilder::selectSource(const QList< Exp
     }
     else if ( targets.length() == 1 ) {
         ExpressionVisitor v(currentContext());
-        v.visitNode(value);
+        v.visitNode(rhs);
         element = SourceType{
             v.lastType(),
             DeclarationPointer(Helper::resolveAliasDeclaration(v.lastDeclaration().data())),
-            v.m_isAlias
+            v.isAlias()
         };
     }
     else if ( ! sources.isEmpty() ) {
@@ -1239,7 +1236,7 @@ DeclarationBuilder::SourceType DeclarationBuilder::selectSource(const QList< Exp
     if ( ! element.type ) {
         // use mixed if none of the previous ways of determining the type worked.
         element.type = AbstractType::Ptr(new IntegralType(IntegralType::TypeMixed));
-        element.declaration = 0;
+        element.declaration = nullptr;
         element.isAlias = false;
     }
     return element;
@@ -1273,25 +1270,25 @@ void DeclarationBuilder::assignToSubscript(SubscriptAst* subscript, const Declar
     }
     ExpressionVisitor targetVisitor(currentContext());
     targetVisitor.visitNode(v);
-    VariableLengthContainer::Ptr cont = VariableLengthContainer::Ptr::dynamicCast(targetVisitor.lastType());
-    if ( cont ) {
-        cont->addContentType(element.type);
+    auto list = ListType::Ptr::dynamicCast(targetVisitor.lastType());
+    if ( list ) {
+        list->addContentType<Python::UnsureType>(element.type);
     }
-    if ( cont && cont->hasKeyType() ) {
+    auto map = MapType::Ptr::dynamicCast(list);
+    if ( map ) {
         if ( subscript->slice && subscript->slice->astType == Ast::IndexAstType ) {
             ExpressionVisitor keyVisitor(currentContext());
             keyVisitor.visitNode(static_cast<IndexAst*>(subscript->slice)->value);
             AbstractType::Ptr key = keyVisitor.lastType();
             if ( key ) {
-                DUChainWriteLocker lock;
-                cont->addKeyType(key);
+                map->addKeyType<Python::UnsureType>(key);
             }
         }
     }
     DeclarationPointer lastDecl = targetVisitor.lastDeclaration();
-    if ( cont && lastDecl ) {
+    if ( list && lastDecl ) {
         DUChainWriteLocker lock;
-        lastDecl->setAbstractType(cont.cast<AbstractType>());
+        lastDecl->setAbstractType(list.cast<AbstractType>());
     }
 }
 
@@ -1416,9 +1413,12 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     
     // check whether this is a type container (list, dict, ...) or just a "normal" class
     if ( Helper::docstringContainsHint(dec, "TypeContainer") ) {
-        VariableLengthContainer* container = new VariableLengthContainer();
+        ListType* container = nullptr;
         if ( Helper::docstringContainsHint(dec, "hasTypedKeys") ) {
-            container->setHasKeyType(true);
+            container = new MapType();
+        }
+        else {
+            container = new ListType();
         }
         type = StructureType::Ptr(container);
     }
@@ -1466,7 +1466,7 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     
     type->setDeclaration(dec);
     dec->setType(type);
-    
+
     openType(type);
     m_currentClassType = type;
 
@@ -1525,10 +1525,11 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
     // if that exists, then this function is a member function
     DeclarationPointer eventualParentDeclaration(currentDeclaration());
     FunctionType::Ptr type(new FunctionType());
-    
-    DUChainWriteLocker lock(DUChain::lock());
-    FunctionDeclaration* dec = eventuallyReopenDeclaration<FunctionDeclaration>(node->name, node->name, FunctionDeclarationType);
-    
+
+    DUChainWriteLocker lock;
+    FunctionDeclaration* dec = eventuallyReopenDeclaration<FunctionDeclaration>(node->name, node->name,
+                                                                                FunctionDeclarationType);
+
     Q_ASSERT(dec->isFunctionDeclaration());
     
     // check for documentation
@@ -1646,12 +1647,12 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
         ExpressionVisitor v(currentContext());
         v.visitNode(node->returns);
         lock.lock();
-        if ( v.lastType() && v.m_isAlias ) {
+        if ( v.lastType() && v.isAlias() ) {
             type->setReturnType(Helper::mergeTypes(type->returnType(), v.lastType()));
             kDebug() << "updated function return type to " << type->toString();
             dec->setType(type);
         }
-        else if ( ! v.m_isAlias ) {
+        else if ( ! v.isAlias()) {
             kDebug() << "not updating function return type because expression is not a type object";
         }
     }
@@ -1660,7 +1661,7 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
     dec->setInSymbolTable(true);
 }
 
-QString DeclarationBuilder::getDocstring(QList< Ast* > body)
+QString DeclarationBuilder::getDocstring(QList< Python::Ast* > body) const
 {
     if ( ! body.isEmpty() && body.first()->astType == Ast::ExpressionAstType 
             && static_cast<ExpressionAst*>(body.first())->value->astType == Ast::StringAstType )
@@ -1747,7 +1748,7 @@ void DeclarationBuilder::adjustExpressionsForTypecheck(Python::ExpressionAst* ad
     second.visitNode(from);
     AbstractType::Ptr hint;
     DeclarationPointer adjust;
-    if ( second.m_isAlias && second.lastType() ) {
+    if ( second.isAlias() && second.lastType() ) {
         hint = second.lastType();
         adjust = first.lastDeclaration();
     }
@@ -1771,7 +1772,7 @@ void DeclarationBuilder::adjustExpressionsForTypecheck(Python::ExpressionAst* ad
 void DeclarationBuilder::visitReturn(ReturnAst* node)
 {
     // Find the type of the object being "return"ed
-    ExpressionVisitor v(currentContext(), editor());
+    ExpressionVisitor v(currentContext());
     v.visitNode(node->value);
     
     if ( node->value ) {
@@ -1835,7 +1836,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
         if ( arg->annotation ) {
             ExpressionVisitor v(currentContext());
             v.visitNode(arg->annotation);
-            if ( v.lastType() && v.m_isAlias ) {
+            if ( v.lastType() && v.isAlias() ) {
                 DUChainWriteLocker lock;
                 argumentType = Helper::mergeTypes(paramDeclaration->abstractType(), v.lastType());
             }
@@ -1854,6 +1855,7 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
 
         kDebug() << "is first:" << isFirst << hasCurrentDeclaration() << currentDeclaration();
         if ( isFirst && hasCurrentDeclaration() && currentContext() && currentContext()->parentContext() ) {
+            DUChainReadLocker lock;
             if ( currentContext()->parentContext()->type() == DUContext::Class ) {
                 argumentType = m_currentClassType.cast<AbstractType>();
                 isFirst = false;
@@ -1898,11 +1900,11 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
         DUChainReadLocker lock;
         AbstractType::Ptr stringType = ExpressionVisitor::typeObjectForIntegralType
                                                         <AbstractType>("str", currentContext());
-        VariableLengthContainer::Ptr dictType = ExpressionVisitor::typeObjectForIntegralType
-                                                        <VariableLengthContainer>("dict", currentContext());
+        auto dictType = ExpressionVisitor::typeObjectForIntegralType
+                                                        <MapType>("dict", currentContext());
         lock.unlock();
         if ( dictType && stringType ) {
-            dictType->addKeyType(stringType);
+            dictType->addKeyType<Python::UnsureType>(stringType);
             visitVariableDeclaration<Declaration>(node->kwarg->argumentName, 0, dictType.cast<AbstractType>());
             type->addArgument(dictType.cast<AbstractType>());
             workingOnDeclaration->setKwarg(type->arguments().size() - 1);
